package api

import (
	"cmf/db/model"
	"cmf/db/repo"
	"cmf/log"
	"errors"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func Craft(r *GinRouter) {
	r.POST("/craft/create", CreateCraft)
	r.POST("/craft/delete/:id", DeleteCraft)
	r.GET("/craft/list", CraftList)
	r.GET("/craft/detail/:id", CraftDetail)
	r.POST("/craft/update", UpdateCraft)
}

func CreateCraft(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var craft model.Craft
	err := c.ShouldBindJSON(&craft)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	craft.CreateTime = time.Now()
	craft.UpdateTime = time.Now()
	return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionCraft, &craft)
}

func DeleteCraft(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionCraft, _id)
}

func CraftList(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	page, size, query := UtilQueryPageSize(c)
	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionCraft,
		Page:        page,
		Size:        size,
		Query:       query,
	})
}

func CraftDetail(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	cate := &model.Craft{}
	found, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: repo.CollectionCraft,
		Query:       repo.Map{"_id": id},
	}, cate)
	if err != nil {
		log.Error(err)
		return nil, err
	}

	if !found {
		return nil, errors.New("未找到该数据")
	}
	return cate, nil
}

func UpdateCraft(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var cate model.Craft
	err := c.ShouldBindJSON(&cate)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	if cate.Id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionCraft, cate.Id.Hex(), &cate)
}
