package api

import (
	"cmf/db/model"
	"cmf/db/repo"
	"cmf/log"
	"errors"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func Decorate(r *GinRouter) {
	r.POST("/decorate/create", CreateDecorate)
	r.POST("/decorate/delete/:id", DeleteDecorate)
	r.GET("/decorate/list", DecorateList)
	r.GET("/decorate/detail/:id", DecorateDetail)
	r.POST("/decorate/update", UpdateDecorate)
}

func CreateDecorate(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var data model.ModelLib
	err := c.ShouldBindJSON(&data)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	data.CreateTime = time.Now()
	data.UpdateTime = time.Now()
	return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionDecorate, &data)
}

func DeleteDecorate(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionDecorate, _id)
}

func DecorateList(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	page, size, query := UtilQueryPageSize(c)
	catsConf, err := GetModelLibCategory(apictx)
	if err != nil {
		return nil, err
	}
	err = ParseCategories(query, apictx.CreateRepoCtx(), catsConf)
	if err != nil {
		return nil, err
	}

	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionDecorate,
		Page:        page,
		Size:        size,
		Query:       query,
		Sort:        bson.M{"createTime": -1},
	})
}

func DecorateDetail(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	box := &model.ModelLib{}
	found, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: repo.CollectionDecorate,
		Query:       repo.Map{"_id": id},
	}, box)
	if err != nil {
		log.Error(err)
		return nil, err
	}

	if !found {
		return nil, errors.New("未找到该数据")
	}
	return box, nil
}

func UpdateDecorate(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var box model.ModelLib
	err := c.ShouldBindJSON(&box)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	if box.Id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionDecorate, box.Id.Hex(), &box)
}
