package api

import (
	"cmf/db/model"
	"cmf/db/repo"
	"cmf/log"
	"errors"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func ModelLib(r *GinRouter) {
	r.POST("/modellib/create", CreateModelLib)
	r.POST("/modellib/delete/:id", DeleteModelLib)
	r.GET("/modellib/list", ModelLibList)
	r.GET("/modellib/detail/:id", ModelLibDetail)
	r.POST("/modellib/update", UpdateModelLib)
}

func CreateModelLib(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var box model.ModelLib
	err := c.ShouldBindJSON(&box)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	box.CreateTime = time.Now()
	box.UpdateTime = time.Now()
	return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionModelLib, &box)
}

func DeleteModelLib(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionModelLib, _id)
}

func ModelLibList(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	page, size, query := UtilQueryPageSize(c)
	catsConf, err := GetModelLibCategory(apictx)
	if err != nil {
		return nil, err
	}
	err = ParseCategories(query, apictx.CreateRepoCtx(), catsConf)
	if err != nil {
		return nil, err
	}

	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionModelLib,
		Page:        page,
		Size:        size,
		Query:       query,
		Sort:        bson.M{"createTime": -1},
	})
}

func ModelLibDetail(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	box := &model.ModelLib{}
	found, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: repo.CollectionModelLib,
		Query:       repo.Map{"_id": id},
	}, box)
	if err != nil {
		log.Error(err)
		return nil, err
	}

	if !found {
		return nil, errors.New("未找到该数据")
	}
	return box, nil
}

func UpdateModelLib(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var box model.ModelLib
	err := c.ShouldBindJSON(&box)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	if box.Id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionModelLib, box.Id.Hex(), &box)
}
